Attribute VB_Name = "modEnumKeysAndValues"
Option Explicit
Option Compare Text
Option Private Module

'''''''''''''''''''''''''''''''''''''''''''''''''''
' Windows API Declares
'''''''''''''''''''''''''''''''''''''''''''''''''''
Private Declare Function RegEnumKeyEx Lib "advapi32.dll" Alias "RegEnumKeyExA" ( _
    ByVal HKey As Long, _
    ByVal dwIndex As Long, _
    ByVal lpName As String, _
    lpcbName As Long, _
    ByVal lpReserved As Long, _
    ByVal lpClass As String, _
    lpcbClass As Long, _
    lpftLastWriteTime As FILETIME) As Long
    
Private Declare Function RegEnumValue Lib "advapi32.dll" Alias "RegEnumValueA" ( _
    ByVal HKey As Long, _
    ByVal dwIndex As Long, _
    ByVal lpValueName As String, _
    lpcbValueName As Long, _
    ByVal lpReserved As Long, _
    lpType As Long, _
    lpData As Byte, _
    lpcbData As Long) As Long
    
Private Declare Function RegOpenKeyEx Lib "advapi32.dll" Alias "RegOpenKeyExA" ( _
    ByVal HKey As Long, _
    ByVal lpSubKey As String, _
    ByVal ulOptions As Long, _
    ByVal samDesired As Long, _
    phkResult As Long) As Long

Private Declare Function RegCloseKey Lib "advapi32.dll" ( _
    ByVal HKey As Long) As Long

Private Type FILETIME
        dwLowDateTime As Long
        dwHighDateTime As Long
End Type

Private Declare Function RegQueryInfoKey Lib "advapi32.dll" Alias "RegQueryInfoKeyA" ( _
    ByVal HKey As Long, _
    ByVal lpClass As String, _
    lpcbClass As Long, _
    ByVal lpReserved As Long, _
    lpcSubKeys As Long, _
    lpcbMaxSubKeyLen As Long, _
    lpcbMaxClassLen As Long, _
    lpcValues As Long, _
    lpcbMaxValueNameLen As Long, _
    lpcbMaxValueLen As Long, _
    lpcbSecurityDescriptor As Long, _
    lpftLastWriteTime As FILETIME) As Long


Private Const REGSTR_MAX_VALUE_LENGTH  As Long = &H100

Private Const KEY_QUERY_VALUE = &H1
Private Const KEY_SET_VALUE = &H2
Private Const KEY_CREATE_SUB_KEY = &H4
Private Const KEY_ENUMERATE_SUB_KEYS = &H8
Private Const KEY_NOTIFY = &H10
Private Const KEY_CREATE_LINK = &H20
Private Const KEY_ALL_ACCESS = &H3F

Private Const REG_CREATED_NEW_KEY = &H1
Private Const REG_OPENED_EXISTING_KEY = &H2

Private Const STANDARD_RIGHTS_ALL = &H1F0000
Private Const SPECIFIC_RIGHTS_ALL = &HFFFF

Private Const REG_OPTION_NON_VOLATILE = 0&
Private Const REG_OPTION_VOLATILE = &H1

Private Const ERROR_SUCCESS = 0&
Private Const ERROR_ACCESS_DENIED = 5
Private Const ERROR_INVALID_DATA = 13&
Private Const ERROR_MORE_DATA = 234 '  dderror
Private Const ERROR_NO_MORE_ITEMS = 259

Private Const S_OK = &H0
Private Const MAX_DATA_BUFFER_SIZE = 1024


Public Function RegistryValueNamesToArray(BaseKey As Long, KeyName As String) As Variant
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' RegistryValueNamesToArray
' This returns an array of the Value Names under KeyName. It returns NULL if an error
' occurs or if KeyName does not exist.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Dim Arr() As String
Dim HKey As Long
Dim Res As Long
Dim N As Long
Dim ValueName As String
Dim KeyTime As FILETIME
Dim NameLen As Long
Dim TType As Long
Dim DData As String * 255
Dim DDataSize As Long
DDataSize = 255
NameLen = 255
ValueName = String$(NameLen, " ")


If RegistryKeyExists(BaseKey:=BaseKey, KeyName:=KeyName) = False Then
    RegistryValueNamesToArray = Null
    Exit Function
End If

Res = RegOpenKeyEx(HKey:=BaseKey, lpSubKey:=KeyName, ulOptions:=0&, samDesired:=KEY_QUERY_VALUE, phkResult:=HKey)
If Res = ERROR_SUCCESS Then
    N = 0
    Res = RegEnumValue(HKey:=HKey, dwIndex:=N, lpValueName:=ValueName, lpcbValueName:=NameLen, lpReserved:=(0&), _
        lpType:=TType, lpData:=ByVal 0&, lpcbData:=DDataSize)
    Do Until Res = ERROR_NO_MORE_ITEMS
        ReDim Preserve Arr(0 To N)
        ValueName = TrimToNull(Text:=ValueName)
        Arr(UBound(Arr)) = ValueName
        N = N + 1
        NameLen = 255
        
        ValueName = String$(NameLen, " ")
        Res = RegEnumValue(HKey:=HKey, dwIndex:=N, lpValueName:=ValueName, lpcbValueName:=NameLen, lpReserved:=(0&), _
            lpType:=TType, lpData:=ByVal 0&, lpcbData:=DDataSize)
    
    Loop
    
Else
    RegCloseKey HKey
    RegistryValueNamesToArray = Null
    Exit Function
End If
RegCloseKey HKey
RegistryValueNamesToArray = Arr

End Function



Public Function RegistrySubKeyNamesToArray(BaseKey As Long, KeyName As String) As Variant
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' RegistrySubKeyNamesToArray
' This function returns an array of strings, each element of which is the name of
' a subkey under KeyName. The function reutrns NULL if an error occurs or if
' KeyName does not exist.
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Dim Arr() As String
Dim HKey As Long
Dim Res As Long
Dim N As Long
Dim SubKeyName As String
Dim KeyTime As FILETIME
Dim NameLen As Long

NameLen = 260
SubKeyName = String$(NameLen, vbNullChar)
If RegistryKeyExists(BaseKey:=BaseKey, KeyName:=KeyName) = False Then
    RegistrySubKeyNamesToArray = Null
End If

Res = RegOpenKeyEx(HKey:=BaseKey, lpSubKey:=KeyName, ulOptions:=0&, samDesired:=KEY_ALL_ACCESS, phkResult:=HKey)
If Res = ERROR_SUCCESS Then
    N = 0
    Res = RegEnumKeyEx(HKey:=HKey, dwIndex:=N, lpName:=SubKeyName, lpcbName:=NameLen, lpReserved:=0&, lpClass:=0&, _
        lpcbClass:=0&, lpftLastWriteTime:=KeyTime)
    Do Until Res = ERROR_NO_MORE_ITEMS
        ReDim Preserve Arr(0 To N)
        SubKeyName = TrimToNull(SubKeyName)
        Arr(UBound(Arr)) = SubKeyName
        NameLen = 260
        SubKeyName = String$(NameLen, vbNullChar)
        N = N + 1
        Res = RegEnumKeyEx(HKey:=HKey, dwIndex:=N, lpName:=SubKeyName, lpcbName:=NameLen, lpReserved:=0&, lpClass:=0&, _
            lpcbClass:=0&, lpftLastWriteTime:=KeyTime)
    Loop
Else
    On Error Resume Next
    RegCloseKey HKey
    RegistrySubKeyNamesToArray = Null
    Exit Function
End If
RegCloseKey HKey
RegistrySubKeyNamesToArray = Arr

End Function

Public Function RegistryNumberOfSubKeys(BaseKey As Long, KeyName As String) As Long
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' RegistryNumberOfSubKeys
' This returns the number of subkeys of KeyName. It returns -1 is an error occurred
' or the key does not exist.
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Dim SubKeyCount As Long
Dim MaxSubKeyLen As Long
Dim MaxValueNameLen As Long
Dim Res As Long
Dim HKey As Long
Dim FT As FILETIME
Dim MaxValueLen As Long

If RegistryKeyExists(BaseKey:=BaseKey, KeyName:=KeyName) = False Then
    RegistryNumberOfSubKeys = -1
    Exit Function
End If

Res = RegOpenKeyEx(HKey:=BaseKey, lpSubKey:=KeyName, ulOptions:=0&, samDesired:=KEY_ALL_ACCESS, phkResult:=HKey)
If Res <> ERROR_SUCCESS Then
    RegistryNumberOfSubKeys = -1
    Exit Function
End If

Res = RegQueryInfoKey( _
    HKey:=HKey, _
    lpClass:=0&, _
    lpcbClass:=0&, _
    lpReserved:=0&, _
    lpcSubKeys:=SubKeyCount, _
    lpcbMaxSubKeyLen:=MaxSubKeyLen, _
    lpcbMaxClassLen:=0&, _
    lpcValues:=0&, _
    lpcbMaxValueNameLen:=MaxValueNameLen, _
    lpcbMaxValueLen:=MaxValueLen, _
    lpcbSecurityDescriptor:=0&, _
    lpftLastWriteTime:=FT)
    
RegCloseKey HKey
If Res = ERROR_SUCCESS Then
    RegistryNumberOfSubKeys = SubKeyCount
Else
    RegistryNumberOfSubKeys = -1
End If

End Function

Public Function RegistryNumberOfValues(BaseKey As Long, KeyName As String) As Long
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' RegistryNumberOfValues
' This returns the number of values under KeyName. It returns -1 is an error occurred
' or the key does not exist.
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Dim SubKeyCount As Long
Dim MaxSubKeyLen As Long
Dim MaxValueNameLen As Long
Dim Res As Long
Dim HKey As Long
Dim FT As FILETIME
Dim MaxValueLen As Long
Dim NumValues As Long
If RegistryKeyExists(BaseKey:=BaseKey, KeyName:=KeyName) = False Then
    RegistryNumberOfValues = -1
    Exit Function
End If

Res = RegOpenKeyEx(HKey:=BaseKey, lpSubKey:=KeyName, ulOptions:=0&, samDesired:=KEY_ALL_ACCESS, phkResult:=HKey)
If Res <> ERROR_SUCCESS Then
    RegistryNumberOfValues = -1
    Exit Function
End If

Res = RegQueryInfoKey( _
    HKey:=HKey, _
    lpClass:=0&, _
    lpcbClass:=0&, _
    lpReserved:=0&, _
    lpcSubKeys:=SubKeyCount, _
    lpcbMaxSubKeyLen:=MaxSubKeyLen, _
    lpcbMaxClassLen:=0&, _
    lpcValues:=NumValues, _
    lpcbMaxValueNameLen:=MaxValueNameLen, _
    lpcbMaxValueLen:=MaxValueLen, _
    lpcbSecurityDescriptor:=0&, _
    lpftLastWriteTime:=FT)
    
RegCloseKey HKey
If Res = ERROR_SUCCESS Then
    RegistryNumberOfValues = NumValues
Else
    RegistryNumberOfValues = -1
End If

End Function


Private Function TrimToNull(Text As String) As String
Dim Pos As Integer
Pos = InStr(1, Text, vbNullChar, vbBinaryCompare)
If Pos > 0 Then
    TrimToNull = Left$(Text, Pos - 1)
Else
    TrimToNull = Text
End If
End Function

Private Function IsArrayAllocated(Arr As Variant) As Boolean
    Dim N As Long
    If IsArray(Arr) = False Then
        IsArrayAllocated = False
        Exit Function
    End If
    On Error Resume Next
    Err.Clear
    N = LBound(Arr, 1)
    If Err.Number = 0 Then
        If LBound(Arr) <= UBound(Arr) Then
            IsArrayAllocated = True
        Else
            IsArrayAllocated = False
        End If
    Else
        IsArrayAllocated = False
    End If
    
End Function



