Attribute VB_Name = "modRegistryKeysAndValues"
Option Explicit


''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' modRegistryKeysAndValues
' By Chip Pearson, chip@cpearson.com, www.cpearson.com
'
' Contains procedures for listing child values and subkeys of a given
' registry key.
' Requires modGetSystemErrorMessageText (www.cpearson.com/Excel/FormatMessage.aspx)
' and modRegistryInclude (www.cpearson.com/Excel/RegistryWorx.aspx)
'
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''



Private Declare Function RegOpenKey Lib "advapi32.dll" Alias "RegOpenKeyA" ( _
    ByVal HKey As Long, _
    ByVal lpSubKey As String, _
    phkResult As Long) As Long

Private Declare Function RegOpenKeyEx Lib "advapi32.dll" Alias "RegOpenKeyExA" ( _
    ByVal HKey As Long, _
    ByVal lpSubKey As String, _
    ByVal ulOptions As Long, _
    ByVal samDesired As Long, _
    phkResult As Long) As Long

Private Declare Function RegQueryValue Lib "advapi32.dll" Alias "RegQueryValueA" ( _
    ByVal HKey As Long, _
    ByVal lpSubKey As String, _
    ByVal lpValue As String, _
    lpcbValue As Long) As Long

Private Declare Function RegSaveKey Lib "advapi32.dll" Alias "RegSaveKeyA" ( _
    ByVal HKey As Long, _
    ByVal lpFile As String, _
    lpSecurityAttributes As SECURITY_ATTRIBUTES) As Long

Private Declare Function RegSetValue Lib "advapi32.dll" Alias "RegSetValueA" ( _
    ByVal HKey As Long, _
    ByVal lpSubKey As String, _
    ByVal dwType As Long, _
    ByVal LPData As String, _
    ByVal cbData As Long) As Long

Private Declare Function RegSetValueEx Lib "advapi32.dll" Alias "RegSetValueExA" ( _
    ByVal HKey As Long, _
    ByVal lpValueName As String, _
    ByVal Reserved As Long, _
    ByVal dwType As Long, _
    LPData As Any, _
    ByVal cbData As Long) As Long         ' Note that if you declare the lpData parameter as String, you must pass it By Value.


Private Declare Function RegCloseKey Lib "advapi32.dll" (ByVal HKey As Long) As Long

Private Declare Function RegEnumKeyEx Lib "advapi32.dll" Alias "RegEnumKeyExA" ( _
    ByVal HKey As Long, _
    ByVal DWIndex As Long, _
    ByVal lpName As String, _
    lpcbName As Long, _
    ByVal lpReserved As Long, _
    ByVal LPClass As String, _
    lpcbClass As Long, _
    lpftLastWriteTime As FILETIME) As Long

Private Declare Function RegEnumValue Lib "advapi32.dll" Alias "RegEnumValueA" ( _
    ByVal HKey As Long, _
    ByVal DWIndex As Long, _
    ByVal lpValueName As String, _
    lpcbValueName As Long, _
    ByVal lpReserved As Long, _
    LPType As Long, _
    LPData As Byte, _
    lpcbData As Long) As Long

Private Declare Function RegQueryInfoKey Lib "advapi32.dll" Alias "RegQueryInfoKeyA" ( _
    ByVal HKey As Long, _
    ByVal LPClass As String, _
    lpcbClass As Long, _
    ByVal lpReserved As Long, _
    lpcSubKeys As Long, _
    lpcbMaxSubKeyLen As Long, _
    lpcbMaxClassLen As Long, _
    lpcValues As Long, _
    lpcbMaxValueNameLen As Long, _
    lpcbMaxValueLen As Long, _
    lpcbSecurityDescriptor As Long, _
    lpftLastWriteTime As FILETIME) As Long

Private Type FILETIME
        dwLowDateTime As Long
        dwHighDateTime As Long
End Type

Private Type SECURITY_ATTRIBUTES
  nLength As Long
  lpSecurityDescriptor As Long
  bInheritHandle As Boolean
End Type


Private Const KEY_ALL_ACCESS = &H3F
Private Const ERROR_SUCCESS = 0&
Private Const ERROR_NO_MORE_ITEMS = 259

Public Function RegistrySubKeyCount(BaseKey As Long, KeyName As String) As Long
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' RegistrySubKeyCount
' Returns the number of subkeys contained under KeyName in BaseKey. This does
' not count the (Defualt) subkey. It does not count nested subkeys. Only direct
' child keys of KeyName are counted. If an error occurs or there are no
' subkeys, the result is an unallocated array. Use IsArrayAllocated to
' determine whether the result array has been allocated and contains data.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Dim HKey As Long
Dim Res As Long
Dim KeyClass As String
Dim KeyClassLen As Long
Dim SubKeyCount As Long
Dim MaxSubKeyNameLen As Long
Dim MaxKeyClassLen As Long
Dim ValuesCount As Long
Dim MaxValueNameLen As Long
Dim MaxValueLen As Long
Dim SecurityLen As Long
Dim LastWriteTime As FILETIME
Dim SysErrNum As Long
Dim SysErrText As String

Const MAX_KEY_CLASS = 1024
Const MAX_KEY_NAME_LEN = 1024

KeyClass = String$(MAX_KEY_CLASS, vbNullChar)
KeyClassLen = MAX_KEY_CLASS
SubKeyCount = 1
MaxSubKeyNameLen = MAX_KEY_NAME_LEN


Res = RegOpenKeyEx(HKey:=BaseKey, lpSubKey:=KeyName, ulOptions:=0&, samDesired:=KEY_ALL_ACCESS, phkResult:=HKey)
If Res <> ERROR_SUCCESS Then
    RegCloseKey HKey
    RegistrySubKeyCount = -1
    SysErrText = GetSystemErrorMessageText(Res)
    Exit Function
End If
    
Res = RegQueryInfoKey(HKey:=HKey, LPClass:=vbNullChar, lpcbClass:=KeyClassLen, lpReserved:=0&, lpcSubKeys:=SubKeyCount, _
    lpcbMaxSubKeyLen:=MaxSubKeyNameLen, lpcbMaxClassLen:=MaxSubKeyNameLen, lpcValues:=ValuesCount, lpcbMaxValueNameLen:=MaxValueNameLen, _
    lpcbMaxValueLen:=0&, lpcbSecurityDescriptor:=0&, lpftLastWriteTime:=LastWriteTime)
    
If Res <> ERROR_SUCCESS Then
    RegCloseKey HKey
    RegistrySubKeyCount = -1
    SysErrText = GetSystemErrorMessageText(Res)
    Exit Function
End If

RegistrySubKeyCount = SubKeyCount
RegCloseKey HKey


End Function

Public Function RegistryValueCount(BaseKey As Long, KeyName As String) As Long
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' RegistryValueCount
' Returns the number of values under KeyName in BaseKey. This does not count
' the (Default) key and does not count nested values. Only direct child values
' of KeyName are counted. If an error occcurs or there are no values, the
' result is an unallocated array. You should use IsArrayAllocated to determine
' whether the array has been allocated and contains data.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Dim HKey As Long
Dim Res As Long
Dim KeyClass As String
Dim KeyClassLen As Long
Dim SubKeyCount As Long
Dim MaxSubKeyNameLen As Long
Dim MaxKeyClassLen As Long
Dim ValuesCount As Long
Dim MaxValueNameLen As Long
Dim MaxValueLen As Long
Dim SecurityLen As Long
Dim LastWriteTime As FILETIME
Dim SysErrNum As Long
Dim SysErrText As String

Const MAX_KEY_CLASS = 1024
Const MAX_KEY_NAME_LEN = 1024

KeyClass = String$(MAX_KEY_CLASS, vbNullChar)
KeyClassLen = MAX_KEY_CLASS
SubKeyCount = 1
MaxSubKeyNameLen = MAX_KEY_NAME_LEN


Res = RegOpenKeyEx(HKey:=BaseKey, lpSubKey:=KeyName, ulOptions:=0&, samDesired:=KEY_ALL_ACCESS, phkResult:=HKey)
If Res <> ERROR_SUCCESS Then
    RegCloseKey HKey
    RegistryValueCount = -1
    SysErrText = GetSystemErrorMessageText(Res)
    Exit Function
End If
    
Res = RegQueryInfoKey(HKey:=HKey, LPClass:=vbNullChar, lpcbClass:=KeyClassLen, lpReserved:=0&, lpcSubKeys:=SubKeyCount, _
    lpcbMaxSubKeyLen:=MaxSubKeyNameLen, lpcbMaxClassLen:=MaxSubKeyNameLen, lpcValues:=ValuesCount, lpcbMaxValueNameLen:=MaxValueNameLen, _
    lpcbMaxValueLen:=0&, lpcbSecurityDescriptor:=0&, lpftLastWriteTime:=LastWriteTime)
    
If Res <> ERROR_SUCCESS Then
    RegCloseKey HKey
    RegistryValueCount = -1
    SysErrText = GetSystemErrorMessageText(Res)
    Exit Function
End If

RegistryValueCount = ValuesCount
RegCloseKey HKey


End Function

Public Function RegistryValuesOfKey(BaseKey As Long, KeyName As String) As String()
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' ValuesOfKey
' This returns an array of strings, each of which is a child value of KeyName.
' Nested values are not returned. Only direct child values are returned. If
' an error occurs or there are no values below KeyName, the function returns
' an unallocated array. You should use IsArrayAllocated to determine whether
' the array is allocated and contains data.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Dim Arr() As String
Dim NumValues As Long
Dim HKey As Long
Dim DWIndex As Long
Dim ValueNameLen As Long
Dim ValueName As String
Dim LPType As Long
Dim LPByte() As Byte
Dim ByteCount As Long
Dim SysErrText As String
Dim SysErrNum As Long
Dim Res As Long
Dim Ndx As Long
Dim LLL As Long
Const MAX_VALUE_NAME = 163830
Const MAX_VALUE_LEN = 1000000

NumValues = RegistryValueCount(BaseKey:=BaseKey, KeyName:=KeyName)
If NumValues <= 0 Then
    Exit Function
End If
ReDim Arr(0 To NumValues - 1)

Res = RegOpenKeyEx(HKey:=BaseKey, lpSubKey:=KeyName, ulOptions:=0&, samDesired:=KEY_ALL_ACCESS, phkResult:=HKey)
If Res <> ERROR_SUCCESS Then
    RegCloseKey HKey
    SysErrText = GetSystemErrorMessageText(Res)
    Exit Function
End If

ValueName = String(MAX_VALUE_NAME, vbNullChar)
ValueNameLen = MAX_VALUE_NAME
ReDim LPByte(0 To MAX_VALUE_LEN)
ByteCount = MAX_VALUE_LEN
DWIndex = 0
Ndx = 0
Res = RegEnumValue(HKey:=HKey, DWIndex:=DWIndex, lpValueName:=ValueName, lpcbValueName:=ValueNameLen, lpReserved:=0&, LPType:=0&, _
    LPData:=ByVal LPByte(0), lpcbData:=LLL)

Do Until Res = ERROR_NO_MORE_ITEMS
    
    Arr(Ndx) = TrimToNull(ValueName)
    ValueNameLen = MAX_VALUE_NAME
    
    ValueName = String(MAX_VALUE_NAME, vbNullChar)
    ReDim LPByte(0 To MAX_VALUE_LEN)
    ByteCount = MAX_VALUE_LEN
    DWIndex = DWIndex + 1
    Res = RegEnumValue(HKey:=HKey, DWIndex:=DWIndex, lpValueName:=ValueName, lpcbValueName:=ValueNameLen, lpReserved:=0&, LPType:=LPType, _
        LPData:=LPByte(0), lpcbData:=ByteCount)
    Ndx = Ndx + 1
    
Loop

RegistryValuesOfKey = Arr
RegCloseKey HKey

End Function

Public Function RegistrySubKeysOfKey(BaseKey As Long, KeyName As String) As String()
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' RegistrySubKeysOfKey
' This returns an array of strings each of which is the name of a subkey of KeyName
' under BaseKey. Nested subkeys are not listed. Only direct child subkeys of KeyName
' are listed. If an error occurs or there are no subkeys, an unallocated array is
' returned. You should use IsArrayAllocated to determine whether the array has been
' allocated and contains data.
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Dim HKey As Long
Dim Res As Long
Dim DWIndex As Long
Dim SubKeyName As String
Dim SubKeyNameLen As Long
Dim LPClass As String
Dim LPClassLen As Long
Dim LastWriteTime As FILETIME
Dim SysErrText As String
Dim Arr() As String
Dim NumSubKeys As String
Dim Ndx As Long

Const MAX_KEY_NAME_LEN = 16000
Const MAX_KEY_CLASS_NAME_LEN = 1024


Res = RegOpenKeyEx(BaseKey, KeyName, 0&, KEY_ALL_ACCESS, HKey)
If Res <> ERROR_SUCCESS Then
    SysErrText = GetSystemErrorMessageText(Res)
    Exit Function
End If

NumSubKeys = RegistrySubKeyCount(BaseKey, KeyName)
If NumSubKeys <= 0 Then
    Exit Function
End If

SubKeyName = String(MAX_KEY_NAME_LEN, vbNullChar)
LPClass = String(MAX_KEY_CLASS_NAME_LEN, vbNullChar)

ReDim Arr(0 To NumSubKeys - 1)
DWIndex = 0
Ndx = 0


SubKeyName = String(MAX_KEY_NAME_LEN, vbNullChar)
LPClass = String(MAX_KEY_CLASS_NAME_LEN, vbNullChar)
SubKeyNameLen = MAX_KEY_NAME_LEN
LPClassLen = MAX_KEY_CLASS_NAME_LEN
Res = RegEnumKeyEx(HKey, DWIndex, SubKeyName, SubKeyNameLen, 0&, LPClass, LPClassLen, LastWriteTime)

Do Until Res = ERROR_NO_MORE_ITEMS
    Arr(Ndx) = TrimToNull(SubKeyName)
    DWIndex = DWIndex + 1
    Ndx = Ndx + 1
    SubKeyName = String(MAX_KEY_NAME_LEN, vbNullChar)
    LPClass = String(MAX_KEY_CLASS_NAME_LEN, vbNullChar)
    SubKeyNameLen = MAX_KEY_NAME_LEN
    LPClassLen = MAX_KEY_CLASS_NAME_LEN

    Res = RegEnumKeyEx(HKey, DWIndex, SubKeyName, SubKeyNameLen, 0&, LPClass, LPClassLen, LastWriteTime)
Loop
RegCloseKey HKey
RegistrySubKeysOfKey = Arr

End Function


Public Function IsArrayAllocated(Arr As Variant) As Boolean
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' IsArrayAllocated
' Returns TRUE or FALSE indicating whether the dynamic
' array Arr has been allocated and contains data. Returns
' True for a static array.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    On Error Resume Next
    IsArrayAllocated = Not IsError(LBound(Arr)) And LBound(Arr) <= UBound(Arr)
End Function

Public Function TrimToNull(S As String) As String
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' TrimToNull
' This returns the characters of S to the left of vbNullChar.
' If vbNullChar is not found in the string, the entire contents
' of the string is returned.
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Dim L As Long
L = InStr(1, S, vbNullChar, vbBinaryCompare)
If L > 0 Then
    TrimToNull = Left(S, L - 1)
Else
    TrimToNull = S
End If
    
End Function

